/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Sample sub extension for debug */

#define IN_EXTENSION
#define DEFINE_DEBUG_MACRO	sub_sample
#define DECLARE_API_POINTERS
#include <waaad/waaad.h>

#include "radius_gw.h"
#include <string.h>
#include <stdlib.h>
#include <errno.h>

int sub_sample_verbosity = 2;

struct sample_state {
	char * conffile;
};

static void * sample_conf_parse(char * conffile)
{
	struct sample_state * ret = NULL;
	
	TRACE_ENTRY("%p", conffile);
	
	CHECK_MALLOC_DO( ret = malloc(sizeof(struct sample_state)), return NULL );
	
	ret->conffile = conffile;
	
	return ret;
}

static void sample_conf_free(void * conf)
{
	TRACE_ENTRY("%p", conf);
	CHECK_PARAMS_DO( conf, );
	free(conf);
	return;
}

static int sample_rad_req(void * conf, sess_id_t ** session, rad_t ** rad_req, msg_t ** diam_fw )
{
	TRACE_ENTRY("%p %p %p %p", conf, session, rad_req, diam_fw);
	return ENOTSUP;
}

static int sample_diam_ans(void * conf, sess_id_t ** session, msg_t ** diam_ans, rad_t ** rad_fw )
{
	TRACE_ENTRY("%p %p %p %p", conf, session, diam_ans, rad_fw);
	return ENOTSUP;
}

int rga_register(int version, waaad_api_t * waaad_api, struct radius_gw_api * api)
{
	TRACE_ENTRY("%d %p", version, api);
	CHECK_PARAMS( waaad_api && api );
	
	if (version != RADIUS_GW_API_VER) {
		log_error("ABI version mismatch, please recompile this extension (%s)\n", __FILE__);
		return EINVAL;
	}
	
	EXTENSION_API_INIT_INTERN( API_MODULE_ALL, "sub_sample", waaad_api );
	
	api->rga_conf_parse_cb = sample_conf_parse;
	api->rga_conf_free_cb  = sample_conf_free;
	api->rga_rad_req_cb    = sample_rad_req;
	api->rga_diam_ans_cb   = sample_diam_ans;
	
	return 0;
}
