/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Header file for the sec_nosec extension. 
 *
 *  This extension provides the "no encryption" mechanism to the waaad daemon.
 * The extension receives a configuration file that must list explicitely all
 * peers to which the peer can connect without additional security (i.e. IPsec
 * or other transparent mechanism is already set).
 * Note that the extension only uses the diameter identity of the peer.
 * To improve security, one should also use the sockaddr parameters.
 *
 *  A better replacement for this extension would be a binding with a IKEv2 process
 * for example, where peers to which a SA is already established are automatically
 * accepted.
 */
 
#define IN_EXTENSION
#define DEFINE_DEBUG_MACRO	sec_nosec
#include <waaad/waaad.h>

#include <pthread.h>
#include <stdlib.h>
#include <string.h>


/* The value of the Inband-Security-Id that this extension provides */
#define SEC_NOSEC_INBAND_SECURITY_ID	0

/* The maximum size of messages we accept. Bigger messages are considered as trash and failure occurs */
#ifndef DIAMETER_MSG_SIZE_MAX
#define DIAMETER_MSG_SIZE_MAX	65536
#endif /* DIAMETER_MSG_SIZE_MAX */

extern int sec_nosec_verbosity;


/**************************************************************************
 * The functions that are provided back to the daemon, as a security module. 
 * See <waaad/security-api.h> 
 **************************************************************************/
int sns_state_change (sec_pss_t newstate, sec_pss_t oldstate, sec_session_t * session, void ** ext_session);
int sns_send_protect (sec_session_t * session, void ** ext_session, void * data, size_t length);
int sns_recv_unprotect (sec_session_t * session, void ** ext_session, void ** data, size_t *length);
int sns_is_supported_peer (char * diamid, struct sockaddr * sa, int * priority);

/**************************************************************************
 * The functions internal to the extension. 
 **************************************************************************/
/* Initialize the hash table where the peers are stored */
int sns_init(void);

/* Add a new peer entry */
int sns_addpeer( char * diameterid, int priority );

/* Find a peer in the table. Return the priority if found, -1 otherwise */
int sns_findpeer( char * diameterid );

/* Parse the configuration file */
int sns_parse(char * conffile);

/* Dump the list to debug */
void sns_dump(void);

/* Destroy the table and free all data */
int sns_fini(void);
